"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require('express');
const router = express.Router();
const controller_1 = require("../controller");
const middleware_1 = require("../middleware");
const config_1 = __importDefault(require("../config"));
const authentication = require('../controller/authentication');
const passportService = require('../services/passport');
const passport = require('passport');
const requireAuth = passport.authenticate('jwt', { session: false });
const requireSignin = passport.authenticate('local', { session: false });
const requireAuthInProduction = function (req, res, next) {
    if (config_1.default.env === 'release') {
        console.log('Release mode required auth');
        requireAuth(req, res, next);
    }
    else {
        console.log('Dev mode. No auth');
        next();
    }
};
router.post('/signin', requireSignin, authentication.signin);
// router.post('/signup', authentication.signup);
router.get('/report', requireAuthInProduction, controller_1.Controller.report);
router.get('/sync-report', requireAuthInProduction, controller_1.Controller.getSyncReports);
router.get('/scale-report', requireAuthInProduction, controller_1.Controller.getScaleReports);
router.get('/removed', requireAuthInProduction, controller_1.Controller.getRemoved);
router.get('/flush', requireAuthInProduction, controller_1.Controller.flush);
router.get('/history', requireAuthInProduction, controller_1.Controller.history);
router.get('/rare-counter', controller_1.Controller.rareCounter);
router.get('/reset-rare-counter', controller_1.Controller.resetRareCounter);
router.get('/tx-coverage', requireAuthInProduction, controller_1.Controller.getTxCoverage);
router.get('/mock', requireAuthInProduction, controller_1.Controller.mock);
router.get('/counted-events', requireAuthInProduction, controller_1.Controller.countedEvents);
router.get('/invalid-ip', controller_1.Controller.invalidIPs);
router.get('/version', controller_1.Controller.version);
router.get('/status', (req, res) => {
    res.status(200).send({ status: 'online', env: config_1.default.env });
});
router.get('/app-versions', requireAuthInProduction, controller_1.Controller.appVersions);
router.post('/joining', middleware_1.Middleware.fieldExistance(['publicKey']), controller_1.Controller.joining);
router.post('/joined', middleware_1.Middleware.fieldExistance(['publicKey', 'nodeId']), controller_1.Controller.joined);
router.post('/active', middleware_1.Middleware.fieldExistance(['nodeId']), controller_1.Controller.active);
router.post('/removed', middleware_1.Middleware.fieldExistance(['nodeId']), controller_1.Controller.removed);
router.post('/heartbeat', middleware_1.Middleware.fieldExistance(['nodeId', 'data']), controller_1.Controller.heartbeat);
router.post('/sync-statement', middleware_1.Middleware.fieldExistance(['nodeId', 'syncStatement']), controller_1.Controller.syncReport);
router.post('/notify-action-started', [middleware_1.Middleware.fieldExistance(['action']), requireAuthInProduction], controller_1.Controller.notifyActionStarted);
module.exports = router;
//# sourceMappingURL=index.js.map