/// <reference types="node" />
import { NodeList, Report, NodeIpInfo, ActiveReport, SyncReport, CrashNodes, MonitorCountedEventMap, NodeInfoAppData } from '../interface/interface';
import { MarkerCount } from '../utils';
declare type TxCoverageData = {
    txId: string;
    count: number;
    timestamp: number;
};
export declare class Node {
    totalTxInjected: number;
    totalTxRejected: number;
    totalTxExpired: number;
    totalProcessed: number;
    avgTps: number;
    maxTps: number;
    rejectedTps: number;
    lastTotalProcessed: number;
    lastTotalTxRejected: number;
    reportInterval: number;
    nodes: NodeList;
    isTimerStarted: boolean;
    crashTimout: number;
    lostNodeIds: Map<string, boolean>;
    syncStatements: {};
    removedNodes: {};
    crashedNodes: {
        [key: string]: CrashNodes;
    };
    history: {};
    counter: number;
    rareEventCounters: {};
    txCoverageMap: {
        [key: string]: TxCoverageData;
    };
    txCoverageCounter: {
        [key: string]: number;
    };
    countedEvents: MonitorCountedEventMap;
    bogonIpCount: any;
    invalidIpCount: any;
    appData: Map<string, NodeInfoAppData>;
    joiningAppData: Map<string, NodeInfoAppData>;
    syncAppData: Map<string, NodeInfoAppData>;
    networkId: string;
    mode: 'forming' | 'processing' | 'safety' | 'recovery' | 'restart' | 'restore' | 'shutdown';
    cycleRecordStart: number;
    cycleRecordCounter: number;
    cycleDuration: number;
    cycleMarkerCount: MarkerCount;
    queryArchiverInterval: NodeJS.Timeout;
    queryArchiverIntervalTime: number;
    constructor();
    private _createEmptyNodelist;
    queryArchiverRetries(): Promise<void>;
    getArchiverCycleRecord(): Promise<unknown>;
    applyArchiverCycleData(cycleRecord: any): void;
    calculateCycleRecordCounter(): {
        cycleRecordCounter: number;
        cycleRecordTimestamp: number;
    };
    checkStandbyNodes(): void;
    checkSyncingNode(): void;
    updateStandbyNodes(): void;
    joining(publicKey: string, nodeIpInfo: NodeIpInfo, appData: NodeInfoAppData): void;
    getExistingActiveNode(nodeId: string, nodeIpInfo: NodeIpInfo): ActiveReport;
    getExistingSyncingNode(nodeId: string, nodeIpInfo: NodeIpInfo): SyncReport;
    getExistingStandbyNode(publicKey: string, nodeIpInfo: NodeIpInfo): string;
    joined(publicKey: string, nodeId: string, nodeIpInfo: NodeIpInfo, appData: NodeInfoAppData): void;
    active(nodeId: string): void;
    checkCrashedBefore(data: any): void;
    removed(nodeId: string): void;
    syncReport(nodeId: any, syncStatement: any): void;
    processTxCoverage(txCoverage: any): void;
    summarizeTxCoverage(): void;
    getTxCoverage(): {
        txCoverageCounter: {
            [key: string]: number;
        };
    };
    heartbeat(nodeId: string, data: ActiveReport): Promise<void>;
    /**
     * Aggregates the incoming counted events and updates the MonitorCountedEventMap in place and returns it
     * @param currentCountedEvents
     * @param countedEvents
     * @param nodeId
     * @returns
     */
    private aggregateMonitorCountedEvents;
    updateAvgAndMaxTps(): void;
    logSummaryToConsole(): void;
    updateRejectedTps(): void;
    checkDeadOrAlive(): void;
    getHistory(): {};
    getRemoved(): any[];
    getRandomNode(): import("../interface/interface").JoinReport;
    getCountedEvents(): MonitorCountedEventMap;
    getAppVersions(): {
        [key: string]: any;
    };
    getInvalidIps(): {
        bogon: any;
        invalid: any;
    };
    resetRareCounters(): void;
    getRareEventCounters(shouldAggregate: any): {};
    report(lastTimestamp?: number): Report;
    getSyncReports(): {};
    getScaleReports(): any[];
    getActiveList(): ActiveReport[];
    createNodeListBackup(filePath: string): void;
    createNetworkStatBackup(filePath: string): void;
    setNodeList(nodes: NodeList): void;
    setNetworkStat(stats: any): void;
    flush(): void;
}
export {};
