declare type Comparator<T, E = T> = (a: E, b: T) => number;
export declare function insertSorted<T>(arr: T[], item: T, comparator?: Comparator<T>): void;
/**
 * Binary search in JavaScript.
 * Returns the index of of the element in a sorted array or (-n-1) where n is the insertion point for the new element.
 *   If the return value is negative use 1-returned as the insertion point for the element
 * Parameters:
 *     arr - A sorted array
 *     el - An element to search for
 *     compare_fn - A comparator function. The function takes two arguments: (el, ae) and returns:
 *       a negative number  if el is less than ae;
 *       a positive number if el is greater than ae.
 *       0 if el is equal to ae;
 *       note that el is the element we are searching for and ae is an array element from the sorted array
 * The array may contain duplicate elements. If there are more than one equal elements in the array,
 * the returned value can be the index of any one of the equal elements.
 *
 * @param arr
 * @param el
 * @param comparator
 */
export declare function binarySearch<T, E = Partial<T>>(arr: T[], el: E, comparator?: Comparator<T, typeof el>): number;
export declare const computeMedian: (arr?: any[], sort?: boolean) => any;
export declare function isIPv6(ip: string): boolean;
/**
 * check if ip is bogon.
 * call getIpArr can throw an error if the format is wrong
 * @param ip
 * @returns
 */
export declare function isBogonIP(ip: any): boolean;
/**
 * check if ip is invalid for lan use.
 * call getIpArr can throw an error if the format is wrong
 * @param ip
 * @returns
 */
export declare function isInvalidIP(ip: any): boolean;
export declare function mapToObjectRecursive(map: Map<string, any> | {
    [key: string]: any;
}): {
    [key: string]: any;
};
export declare class MarkerCount {
    private nodeMap;
    private markerCount;
    private heap;
    private possibleNewMarker;
    constructor();
    note(nodeId: string, marker: string): void;
    increment(marker: string): void;
    updateNodeMarker(oldMarker: string, newMarker: string): void;
    verifyMarker(marker: string): boolean;
    getCorrectMarker(): string;
    heapify(): void;
}
export declare function compareIPs(ip1: any, ip2: any, type?: string): 1 | -1 | 0;
export {};
